import re
import numpy as np
import pandas as pd
import scipy.io.wavfile as wav
from python_speech_features import *
import os
import matplotlib.pyplot as plt
from sklearn.model_selection import StratifiedKFold, KFold
from sklearn.preprocessing import StandardScaler,MinMaxScaler
import pickle

'''待解决的问题
1.测试数据是自己分的，应该使用10折交叉验证
2.直接从训练集里面取了和异常数据相等的数据量
3.验证集也是用StratifiedKFold函数按照原比例抽取的，并未按照患者分层，若按患者分层则无需考虑此内容，
  或者如果取的话，把该患者的所有数据都取出来(而重复患者有多少段数据也不知道，最好后期是弄成患者互斥的)
4.是否需要下采样？这个需要看mfcc的内部写法
5.train_abnormal_124_3.wav只有2s，换成train_abnormal_124_2.wav   train_abnormal_162_6.wav->train_abnormal_162_5.wav
'''


def extract_mfcc(wav_arr, sample_rate):
    mfcc_feat = mfcc(wav_arr, sample_rate)
    energy = np.sqrt(wav_arr)
    mfcc_feat = np.stack(mfcc_feat, energy)
    delta1 = delta(mfcc_feat, 1)
    delta2 = delta(delta1, 1)
    mfcc_feat = np.stack(mfcc_feat, delta1, delta2)
    return mfcc_feat.T

def get_mfcc(data, fs):
    #默认提取的特征维度是13，通常的做法是将该特征进行一阶差分和二阶差分，并将结果进行合并。
    wav_feature =  mfcc(data, fs)
    d_mfcc_feat = delta(wav_feature, 1)
    d_mfcc_feat2 = delta(wav_feature, 2)
    # print("delta1_shape:{:},delta2_shape:{:}".format(d_mfcc_feat.shape, d_mfcc_feat2.shape))
    feature = np.hstack((wav_feature, d_mfcc_feat, d_mfcc_feat2))
    return feature

def dataset_getmfcc(dataset, sample_rate):
    # 计算dataset的mfcc特征
    print("\033[32m正在提取MFCC特征...\033[0m")   #1199-12s的长度  5s是499
    mfcc_set = np.zeros((dataset.shape[1], 499, 39))
    # print(dataset.shape)
    for i in range(dataset.shape[1]):
        mfcc_set[i] = get_mfcc(dataset[:,i], sample_rate)
        # print("sample_rate:{:d},sig_shape:{:},MFCC_shape:{:}".format(sample_rate, signal.shape, mfcc_set[i].shape))
    print("\033[32m特征提取完毕...\033[0m")
    return mfcc_set


if __name__ == "__main__":
    '''path = "E:/Postgraduate work/Literature/PCG/Other's/data/2020.8.16_data/new_wav_8_26/train"  # 需要是远端的数据路径
    for root, dirs, files in os.walk(path):
        #加载label文件
        csv_file = [file for file in files if file.endswith('.csv')]
        csv_data = load_csv(os.path.join(root,csv_file[0]))
        k_nor_train, k_nor_test, k_abnor_train, k_abnor_test = re_distribute_data(10)
        extract_data_by_number(csv_data, k_nor_train[1], "normal")'''



#!/usr/bin/python
# -*- coding: utf-8 -*-
import sys
# import codecs
# sys.stdout = codecs.getwriter("utf-8")(sys.stdout.detach())  # 这一句让中文能够写到txt文件中
import yaml
import io
import numpy as np
from release_test.CAD_Recognition.data_load import  load_data, _load_data, _data_normalization, data_normalization, _k_folds
from release_test.CAD_Recognition.features import dataset_getmfcc
import os
import tensorflow as tf
from keras import backend as KTF
# import keras.backend.tensorflow_backend as KTF
import keras
from keras import regularizers, initializers
from keras.models import Sequential, load_model
from keras.layers import Dense, Dropout, Activation, Flatten, Conv2D, MaxPooling2D, \
    BatchNormalization, LSTM, Reshape, Conv1D, AveragePooling1D, pooling
from keras.layers.pooling import GlobalAveragePooling1D
from keras.optimizers import RMSprop, Adam
from keras.callbacks import ReduceLROnPlateau, ModelCheckpoint
from keras.utils import multi_gpu_model
import matplotlib.pyplot as plt
from sklearn.metrics import confusion_matrix, classification_report
import time
import math
from tensorflow.contrib.opt import AdamWOptimizer
import warnings
import pickle

def prob_cal(prediction, class_, index, ratio):
    # 取正常对应的输出段的输出概率，然后求均值。
    class_ = 0 if class_ == 0 else 1
    prob_scope = prediction[index * ratio: (index + 1) * ratio]
    # pred_scope = pred[i * ratio: (i + 1) * ratio]  #保留按照索引取正常或者异常的片段的概率
    # prob_index = np.where(pred_scope == 0)
    prob = np.mean(prob_scope[:, class_]) * 100
    prob = 99.99 if prob == 100 else prob
    return prob

def computing_resource(_config):
    """
    计算资源分配
    :param _config: yaml配置文件
    """
    # export CUDA_VISIBLE_DEVICES = n
    gpu_options = tf.GPUOptions(per_process_gpu_memory_fraction=_config["gpu_fraction"])
    config = tf.ConfigProto(intra_op_parallelism_threads=_config["threads_num"],
                            device_count={'CPU': _config["CPU_num"], 'GPU': _config["GPU_num"]},
                            allow_soft_placement=True,
                            gpu_options=gpu_options
                            )
    session = tf.Session(config=config)
    KTF.set_session(session)

def plot_history(history):
    print(history.history.keys())
    plt.clf()
    plt.title(u'Accuracy')
    plt.ylabel('Accuracy')
    plt.xlabel('Epoch')
    plt.plot(history.history['acc'])
    plt.plot(history.history['val_acc'])
    plt.legend(['training', 'valivation'], loc='upper left')
    plt.show()

    plt.title(u'Loss')
    plt.ylim(0, 3)
    plt.ylabel('Model loss')
    plt.xlabel('Epoch')
    plt.plot(history.history['loss'])
    plt.plot(history.history['val_loss'])
    plt.legend(['training', 'valivation'], loc='upper left')
    plt.show()

def pcg_metrics_cal(labels, prediction, mode, y_true, isprint=False, patient=False):
    #分条数据，比例为5:1
    # 1.分条数据，比例为3；分患者数据，比例为9
    section = 5
    ratio = section*3 if patient else section
    # 2.获得数据标签；获得患者标签
    true_index = np.arange(0, labels.shape[0], ratio)
    data_patient_label = labels.argmax(axis=1)[true_index]

    # 3.统计每条3份数据的的类别占比，获得预测标签；统计整个患者的(section*3)段数据的的类别占比，获得预测标签
    pred = prediction.argmax(axis=1)
    data_pred =  np.zeros(( int(pred.shape[0]/ratio) ))
    normal_num = np.zeros(( int(pred.shape[0]/ratio) ))  #正常的条数
    # 每条数据或者每个患者的正常数据条数
    for i in range(int(pred.shape[0]/ratio)):
        pred_scope = pred[i*ratio : (i+1)*ratio]
        normal_num[i] = np.sum(pred_scope == 0)   #计算pred_scope中的0的个数
        data_pred[i] = 0 if normal_num[i]>=math.ceil(ratio/2) else 1
    # 4.计算指标
    matrix = confusion_matrix(data_patient_label, data_pred, labels=[0,1])  #这里的标签需要更改
    TP, FN, FP, TN = matrix[0,0], matrix[0,1], matrix[1,0], matrix[1,1]
    Accuracy = (TP + TN) / np.sum(matrix) * 100
    Sensitivity = (TP / (TP + FN)) * 100            #敏感性
    Specificity = (TN / (TN + FP)) * 100            #特异性
    Precision = (TP / (TP + FP)) * 100              #查准率(准确率)
    Recall = (TP / (TP + FN)) * 100                 #查全率(召回率)
    G_mean = np.sqrt(Sensitivity * Specificity)     #几何平均分(敏感性,特异性赋予了相同权重，后面可以改为更侧重敏感性)
    F1 = 2 * (Precision * Recall)/(Precision + Recall)
    with open("log.txt", "a") as f:
        print(mode+"===>\n"+"matrix:\n{:}\nAccuracy:{:.3f}%\nSensitivity:{:.3f}%\n"
            "Specificity:{:.3f}%\nPrecision:{:.3f}%\nRecall:{:.3f}%\nG_mean:{:.3f}\nF1:{:.3f}".
          format(matrix,Accuracy,Sensitivity, Specificity, Precision, Recall, G_mean, F1),file=f)
    print("\033[32m"+ mode +"=========>\n"+ "matrix:\n{:}\nAccuracy:{:.3f}%\nSensitivity:{:.3f}%\n"
          "Specificity:{:.3f}%\nPrecision:{:.3f}%\nRecall:{:.3f}%\nG_mean:{:.3f}\nF1:{:.3f}\033[0m".
        format(matrix, Accuracy, Sensitivity, Specificity, Precision, Recall, G_mean, F1))
    # 5.转换为三分类，并输出结果
    # 5.1患者判断
    if patient:
        threshold = [2,5]  #2,5
        if isprint:  #改为3分类
            status_list = ['正常', '轻风险', '异常'] #状态
            pred_triple_class = np.zeros(( y_true.shape[0] ))  #真实类别
            #动态记录阈值
            mild_threshold, normal_threshold, normal_prob, mild_prob, severe_prob = np.zeros(1), np.zeros(1), np.zeros(1), np.zeros(1), np.zeros(1)
            #判断该患者是：正常，轻风险，异常(基于二分的结果，正常不进入轻风险和异常判断)
            for i in range(data_pred.shape[0]):
                if data_pred[i] == 0:
                    pred_triple_class[i] = 0   #正常直接判断为正常,相信二分类器
                else:
                    if normal_num[i]<threshold[0]:
                        pred_triple_class[i] = 2
                    elif (normal_num[i] >= threshold[0]) & (normal_num[i] <= threshold[1]):
                        pred_triple_class[i] = 1
                    elif normal_num[i] > threshold[1]:
                        if data_pred[i] == 1:
                            pred_triple_class[i] = 2  #二分若为异常，则不允许判为正常(实际上此处可以对判断正常的概率有一定提升，但是为了安全起见，相信二分类器)
                        else:
                            pred_triple_class[i] = 0
                pred_status = status_list[int(pred_triple_class[i])]
                true_status = status_list[int(y_true[i])]
                if pred_status == true_status == '正常':
                    #取正常对应的输出段的输出概率，然后求均值。
                    prob = prob_cal(prediction, pred_triple_class[i], i, ratio)
                    print("\033[32m患者{}识别结果为:{:s},真实结果为:{:s},正常概率为:{:.2f}%\033[0m".
                          format(i+1, pred_status, true_status, prob))
                elif pred_status == true_status == '异常':
                    prob = prob_cal(prediction, pred_triple_class[i], i, ratio)
                    normal_prob = np.append(normal_prob, prob)
                    print("\033[34m患者{}识别结果为:{:s},真实结果为:{:s},异常概率为:{:.2f}%\033[0m".
                          format(i+1, pred_status, true_status, prob))
                elif (pred_status=='轻风险') & (true_status=='轻风险'):
                    prob = prob_cal(prediction, pred_triple_class[i], i, ratio)
                    normal_prob = np.append(normal_prob, prob)
                    print("\033[34m患者{}识别结果为:{:s},真实结果为:{:s},轻风险概率为:{:.2f}%\033[0m".
                          format(i+1, pred_status, true_status, prob))
                else:
                    prob = prob_cal(prediction, pred_triple_class[i], i, ratio)
                    print("\033[31m患者{}识别结果为:{:s},真实结果为:{:s},异常概率为:{:.2f}\033[0m".
                          format(i+1, pred_status, true_status, prob))
                if true_status=='正常':
                    normal_threshold = np.append(normal_threshold, normal_num[i])
                    normal_prob = np.append(normal_prob, prob)
                elif true_status=='轻风险':
                    mild_threshold = np.append(mild_threshold, normal_num[i])
                    mild_prob = np.append(mild_prob, prob)
                elif true_status=="异常":
                    severe_prob = np.append(severe_prob, prob)

            normal_threshold = np.delete(normal_threshold, 0)
            mild_threshold = np.delete(mild_threshold,0)
            normal_prob = np.delete(normal_prob, 0)
            mild_prob = np.delete(mild_prob, 0)
            severe_prob = np.delete(severe_prob, 0)
            triple_matrix = confusion_matrix(y_true, pred_triple_class, labels=[0,1,2])
            print("患者3分类矩阵:matrix:\n{:}".format(triple_matrix))
            print("patient_classification_report:\n{}".format(classification_report(pred_triple_class, y_true,labels=[0,1,2],target_names=["normal","mild","abnormal"])))
            return mild_threshold, normal_threshold, normal_prob, mild_prob, severe_prob, data_pred
    # 5.2数据判断，判断数据是：正常，异常
    else:
        if isprint:
            status_list = ['正常', '异常']
            for i in range(data_pred.shape[0]):
                pred_status = status_list[int(data_pred[i])]
                true_status = status_list[int(data_patient_label[i])]
                if pred_status == true_status == '正常':
                    print("\033[32m数据{}识别结果为:{:s},真实结果为:{:s}\033[0m".format(i+1, pred_status, true_status))
                elif pred_status == true_status == '异常':
                    print("\033[34m数据{}识别结果为:{:s},真实结果为:{:s}\033[0m".format(i+1, pred_status, true_status))
                else:
                    print("\033[31m数据{}识别结果为:{:s},真实结果为:{:s}\033[0m".format(i+1, pred_status, true_status))
    return np.array([Accuracy, Sensitivity, Specificity, Precision, Recall, G_mean, F1]).reshape((1,7))

def ecg_metrics_cal(labels, prediction, mode, y_true, isprint=False, patient=False):
    # 分条数据，比例为5:1
    # 1.分条数据，比例为3；分v患者数据，比例为9
    section = 5
    ratio = section * 3 if patient else section
    # 2.获得数据标签；获得患者标签
    true_index = np.arange(0, labels.shape[0], ratio)
    data_patient_label = labels.argmax(axis=1)[true_index]

    # 3.统计每条3份数据的的类别占比，获得预测标签；统计整个患者的(section*3)段数据的的类别占比，获得预测标签
    pred = prediction.argmax(axis=1)
    data_pred = np.zeros((int(pred.shape[0] / ratio)))
    normal_num = np.zeros((int(pred.shape[0] / ratio)))  # 正常的条数
    # 每条数据或者每个患者的正常数据条数
    for i in range(int(pred.shape[0] / ratio)):
        pred_scope = pred[i * ratio: (i + 1) * ratio]
        normal_num[i] = np.sum(pred_scope == 0)  # 计算pred_scope中的0的个数
        data_pred[i] = 0 if normal_num[i] >= math.ceil(ratio / 2) else 1
    # 4.计算指标
    matrix = confusion_matrix(data_patient_label, data_pred, labels=[0, 1])  # 这里的标签需要更改
    TP, FN, FP, TN = matrix[0, 0], matrix[0, 1], matrix[1, 0], matrix[1, 1]
    Accuracy = (TP + TN) / np.sum(matrix) * 100
    Sensitivity = (TP / (TP + FN)) * 100  # 敏感性
    Specificity = (TN / (TN + FP)) * 100  # 特异性
    Precision = (TP / (TP + FP)) * 100  # 查准率(准确率)
    Recall = (TP / (TP + FN)) * 100  # 查全率(召回率)
    G_mean = np.sqrt(Sensitivity * Specificity)  # 几何平均分(敏感性,特异性赋予了相同权重，后面可以改为更侧重敏感性)
    F1 = 2 * (Precision * Recall) / (Precision + Recall)
    with open("log.txt", "a") as f:
        print(mode + "===>\n" + "matrix:\n{:}\nAccuracy:{:.3f}%\nSensitivity:{:.3f}%\n"
                                "Specificity:{:.3f}%\nPrecision:{:.3f}%\nRecall:{:.3f}%\nG_mean:{:.3f}\nF1:{:.3f}".
              format(matrix, Accuracy, Sensitivity, Specificity, Precision, Recall, G_mean, F1), file=f)
    print("\033[32m" + mode + "=========>\n" + "matrix:\n{:}\nAccuracy:{:.3f}%\nSensitivity:{:.3f}%\n"
                                               "Specificity:{:.3f}%\nPrecision:{:.3f}%\nRecall:{:.3f}%\nG_mean:{:.3f}\nF1:{:.3f}\033[0m".
          format(matrix, Accuracy, Sensitivity, Specificity, Precision, Recall, G_mean, F1))
    # 5.转换为三分类，并输出结果
    # 5.1患者判断
    if patient:
        threshold = [2, 5]  # 2,5
        if isprint:  # 改为3分类
            status_list = ['正常', '轻风险', '异常']  # 状态
            pred_triple_class = np.zeros((y_true.shape[0]))  # 真实类别
            # 动态记录阈值
            mild_threshold, normal_threshold, normal_prob, mild_prob, severe_prob =\
                np.zeros(1), np.zeros(1), np.zeros(1), np.zeros(1), np.zeros(1)
            # 判断该患者是：正常，轻风险，异常(基于二分的结果，正常不进入轻风险和异常判断)
            for i in range(data_pred.shape[0]):
                if data_pred[i] == 0:
                    pred_triple_class[i] = 0  # 正常直接判断为正常,相信二分类器
                else:
                    if normal_num[i] < threshold[0]:
                        pred_triple_class[i] = 2
                    elif (normal_num[i] >= threshold[0]) & (normal_num[i] <= threshold[1]):
                        pred_triple_class[i] = 1
                    elif normal_num[i] > threshold[1]:
                        if data_pred[i] == 1:
                            pred_triple_class[i] = 2  # 二分若为异常，则不允许判为正常(实际上此处可以对判断正常的概率有一定提升，但是为了安全起见，相信二分类器)
                        else:
                            pred_triple_class[i] = 0
                pred_status = status_list[int(pred_triple_class[i])]
                true_status = status_list[int(y_true[i])]
                if pred_status == true_status == '正常':
                    # 取正常对应的输出段的输出概率，然后求均值。
                    prob = prob_cal(prediction, pred_triple_class[i], i, ratio)
                    print("\033[32m患者{}识别结果为:{:s},真实结果为:{:s},正常概率为:{:.2f}%\033[0m".
                          format(i + 1, pred_status, true_status, prob))
                elif pred_status == true_status == '异常':
                    prob = prob_cal(prediction, pred_triple_class[i], i, ratio)
                    normal_prob = np.append(normal_prob, prob)
                    print("\033[34m患者{}识别结果为:{:s},真实结果为:{:s},异常概率为:{:.2f}%\033[0m".
                          format(i + 1, pred_status, true_status, prob))
                elif (pred_status == '轻风险') & (true_status == '轻风险'):
                    prob = prob_cal(prediction, pred_triple_class[i], i, ratio)
                    normal_prob = np.append(normal_prob, prob)
                    print("\033[34m患者{}识别结果为:{:s},真实结果为:{:s},轻风险概率为:{:.2f}%\033[0m".
                          format(i + 1, pred_status, true_status, prob))
                else:
                    prob = prob_cal(prediction, pred_triple_class[i], i, ratio)
                    print("\033[31m患者{}识别结果为:{:s},真实结果为:{:s},异常概率为:{:.2f}\033[0m".
                          format(i + 1, pred_status, true_status, prob))
                if true_status == '正常':
                    normal_threshold = np.append(normal_threshold, normal_num[i])
                    normal_prob = np.append(normal_prob, prob)
                elif true_status == '轻风险':
                    mild_threshold = np.append(mild_threshold, normal_num[i])
                    mild_prob = np.append(mild_prob, prob)
                elif true_status == "异常":
                    severe_prob = np.append(severe_prob, prob)
                ######################轻风险二分类效果查看
            print("\033[35m\n\n轻风险二分类效果查看:\033[0m")
            for i in range(data_pred.shape[0]):
                true_status = status_list[int(y_true[i])]
                if true_status == "轻风险":
                    if data_pred[i] == 0:
                        view_status = "正常"
                        print("\033[32m识别结果为:{:s},真实结果为:{:s}\033[0m".format( view_status, true_status))
                    else:
                        view_status = "异常"
                        print("\033[31m识别结果为:{:s},真实结果为:{:s}\033[0m".format( view_status, true_status))
                ######################轻风险二分类效果查看
            normal_threshold = np.delete(normal_threshold, 0)
            mild_threshold = np.delete(mild_threshold, 0)
            normal_prob = np.delete(normal_prob, 0)
            mild_prob = np.delete(mild_prob, 0)
            severe_prob = np.delete(severe_prob, 0)
            triple_matrix = confusion_matrix(y_true, pred_triple_class, labels=[0, 1, 2])
            print("患者3分类矩阵:matrix:\n{:}".format(triple_matrix))
            print("patient_classification_report:\n{}".format(
                classification_report(pred_triple_class, y_true, labels=[0, 1, 2],
                                      target_names=["normal", "mild", "abnormal"])))
            return mild_threshold, normal_threshold, normal_prob, mild_prob, severe_prob, data_pred
    # 5.2数据判断，判断数据是：正常，异常
    else:
        if isprint:
            status_list = ['正常', '异常']
            for i in range(data_pred.shape[0]):
                pred_status = status_list[int(data_pred[i])]
                true_status = status_list[int(data_patient_label[i])]
                if pred_status == true_status == '正常':
                    print("\033[32m数据{}识别结果为:{:s},真实结果为:{:s}\033[0m".format(i + 1, pred_status, true_status))
                elif pred_status == true_status == '异常':
                    print("\033[34m数据{}识别结果为:{:s},真实结果为:{:s}\033[0m".format(i + 1, pred_status, true_status))
                else:
                    print("\033[31m数据{}识别结果为:{:s},真实结果为:{:s}\033[0m".format(i + 1, pred_status, true_status))
    return np.array([Accuracy, Sensitivity, Specificity, Precision, Recall, G_mean, F1]).reshape((1, 7))

def sub1_model_test(root1, x_test_name, y_test_label, y_test_true_label, scaler, model_path=None):
    # 1.加载测试数据
    # f = io.open('_config.yaml', 'r', encoding="utf-8")
    # _config = yaml.load(f)
    print("进入PCG测试")
    x_test, y_test, sample_rate = load_data(root1, x_test_name, y_test_label, 5)
    # 2.提取mfcc特征
    x_test = dataset_getmfcc(x_test, sample_rate)
    # 3.特征归一化
    x_test, _ = data_normalization(scaler, raw_train=0, raw_test=x_test)
    x_test = x_test[:, :, :, np.newaxis] # 维度变换
    y_test = keras.utils.to_categorical(y_test, 2)  #独热编码
    # 4.加载模型
    if model_path!=None:
        model = load_model(model_path)
    # 5.前向预测
    y_pred = model.predict(x_test)
    print("\033[32m数据测试\033[0m")
    pcg_metrics_cal(y_test, y_pred, "test", y_test_true_label, isprint=False, patient=False)
    print("\033[32m患者测试\033[0m")
    mild_threshold, normal_threshold, normal_prob, mild_prob, severe_prob, double_results = \
        pcg_metrics_cal(y_test, y_pred, "test", y_test_true_label, isprint=True, patient=True)
    return double_results

def sub2_model_test(root2, x_test_name, y_test_label, y_test_true_label, scaler, model_path=None):
    # 1.加载测试数据
    # f = io.open('_config.yaml', 'r', encoding="utf-8")
    # _config = yaml.load(f)
    print("进入ECG测试")
    x_test, y_test, sample_rate = _load_data(root2, x_test_name, y_test_label, 5)
    # 2.特征归一化
    x_test = x_test.transpose()
    x_test, _ = _data_normalization(scaler, raw_train=0, raw_test=x_test)
    x_test = x_test[:, :, np.newaxis]  # 维度变换
    y_test = keras.utils.to_categorical(y_test, 2)  # 独热编码
    # 3.加载模型
    if model_path != None:
        model = load_model(model_path)
    # 4.前向预测
    y_pred = model.predict(x_test)
    print("\033[32m数据测试\033[0m")
    ecg_metrics_cal(y_test, y_pred, "test", y_test_true_label, isprint=False, patient=False)
    print("\033[32m患者测试\033[0m")
    mild_threshold, normal_threshold, normal_prob, mild_prob, severe_prob, double_class_results = \
        ecg_metrics_cal(y_test, y_pred, "test", y_test_true_label,isprint=True, patient=True)
    return double_class_results

def threshold_test(data_path, model_list, scaler1, scaler2):
    # 分别用不同的k_folds来取出轻风险不同标签的验证数据，但是患者是相同的，只是轻风险标签不同，将数据分别送入两个子分类器进行分类。
    # f = io.open('_config.yaml', 'r', encoding="utf-8")
    # _config = yaml.load(f)
    all_metrics = np.zeros((1, 7))
    matrix = np.zeros((3,3))
    '''model_list = [["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-0_val_loss-0.59_epoch-40_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_0val_loss-0.40_epoch-239_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-1_val_loss-0.38_epoch-06_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_1val_loss-0.30_epoch-35_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-2_val_loss-0.27_epoch-06_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_2val_loss-0.43_epoch-234_ECG_model.hdf5",],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-3_val_loss-0.52_epoch-07_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_3val_loss-0.48_epoch-243_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-4_val_loss-0.50_epoch-02_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_4val_loss-0.52_epoch-118_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-5_val_loss-0.34_epoch-08_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_5val_loss-0.37_epoch-181_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-6_val_loss-0.40_epoch-09_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_6val_loss-0.38_epoch-62_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-7_val_loss-0.26_epoch-04_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_7val_loss-0.29_epoch-327_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-8_val_loss-0.32_epoch-07_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_8val_loss-0.44_epoch-232_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-9_val_loss-0.53_epoch-02_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_9val_loss-0.48_epoch-34_ECG_model.hdf5"]]'''
    root_1, x_kf_train_1, y_kf_train_1, x_kf_valid_1, y_kf_valid_1, y_kf_true_train_1, y_kf_true_valid_1 = \
        _k_folds(data_path, 10, sub=1)
    root_2, x_kf_train_2, y_kf_train_2, x_kf_valid_2, y_kf_valid_2, y_kf_true_train_2, y_kf_true_valid_2 = \
        _k_folds(data_path, 10, sub=2)
    for i in range(0, 10):
        print("\033[31mFolds:{:d}===========================================>\033[0m".format(i + 1))
        sub_1_results = sub1_model_test(root_1, x_kf_valid_1[i], y_kf_valid_1[i], y_kf_true_valid_1[i], scaler1, model_list[0])
        sub_2_results = sub2_model_test(root_2, x_kf_valid_2[i], y_kf_valid_2[i], y_kf_true_valid_2[i], scaler2, model_list[1])
        this_martix = decision_fusion(y_kf_true_valid_1[i], sub_1_results, sub_2_results)
        matrix += this_martix
        if i == 1: break

    # 11.交叉验证训练指标计算
    '''all_metrics = np.delete(all_metrics, 0, axis=0)  # 删掉0行
    macro_metrics = np.mean(all_metrics, axis=0)
    macro_F1 = 2 * (macro_metrics[3] * macro_metrics[4]) / (macro_metrics[3] + macro_metrics[4])
    print("\033[32mmacro results:\nAccuracy:{:.3f}%\nSensitivity:{:.3f}%\nSpecificity:{:.3f}%\nPrecision:{:.3f}%\nRecall:{:.3f}%\nG_mean:{:.3f}\nF1:{:.3f}\033[0m".
        format(macro_metrics[0], macro_metrics[1], macro_metrics[2], macro_metrics[3], macro_metrics[4],
               macro_metrics[5], macro_F1))'''
    print("交叉验证所有矩阵之和:matrix:\n{:}".format(matrix))

def decision_fusion(y_true, sub_1_results, sub_2_results):
    final_results = np.zeros(1)
    for i in range(sub_1_results.shape[0]):
        #1.相同则不变
        if sub_1_results[i] == sub_2_results[i] == 0:
            final_results = np.append(final_results, 0)
        elif sub_1_results[i] == sub_2_results[i] == 1:
            final_results = np.append(final_results, 2)
        #2.PCG异常，ECG正常则判定为轻风险
        elif (sub_1_results[i]==1) and (sub_2_results[i]==0):
            final_results = np.append(final_results, 1)
        #3.PCG正常，ECG异常则相信PCG分类
        elif (sub_1_results[i]==0) and (sub_2_results[i]==1):
            final_results = np.append(final_results, 0)
    final_results = np.delete(final_results, 0)
    triple_matrix = confusion_matrix(y_true, final_results, labels=[0, 1, 2])
    print("融合患者3分类矩阵:matrix:\n{:}".format(triple_matrix))
    print("patient_classification_report:\n{}".format(classification_report(final_results, y_true,
        labels=[0, 1, 2], target_names=["normal", "mild", "abnormal"])))
    return triple_matrix

def dataset_test(data_path, model_list, scaler1, scaler2):
    print("进入测试")
    with warnings.catch_warnings():
        warnings.filterwarnings('ignore', category=UserWarning)
        warnings.filterwarnings('ignore', category=RuntimeWarning)
        threshold_test(data_path, model_list, scaler1, scaler2)

if __name__ == "__main__":
    '''model_list = [["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-0_val_loss-0.59_epoch-40_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_0val_loss-0.40_epoch-239_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-1_val_loss-0.38_epoch-06_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_1val_loss-0.30_epoch-35_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-2_val_loss-0.27_epoch-06_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_2val_loss-0.43_epoch-234_ECG_model.hdf5",],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-3_val_loss-0.52_epoch-07_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_3val_loss-0.48_epoch-243_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-4_val_loss-0.50_epoch-02_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_4val_loss-0.52_epoch-118_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-5_val_loss-0.34_epoch-08_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_5val_loss-0.37_epoch-181_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-6_val_loss-0.40_epoch-09_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_6val_loss-0.38_epoch-62_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-7_val_loss-0.26_epoch-04_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_7val_loss-0.29_epoch-327_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-8_val_loss-0.32_epoch-07_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_8val_loss-0.44_epoch-232_ECG_model.hdf5"],
                  ["E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-9_val_loss-0.53_epoch-02_PCG_model.hdf5","E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/release_test/Model/9.21/5_percentage/_folds_9val_loss-0.48_epoch-34_ECG_model.hdf5"]]'''
    # f = open("_config.yaml", 'r', encoding="utf-8")
    # _config = yaml.load(f)
    # computing_resource(_config)
    # model_fusion()

    '''scaler1 = pickle.load(open('E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/Model/scaler/scaler_8_30_triple.pkl', 'rb'))
    scaler2 = pickle.load(open('E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/Model/scaler/scaler_9_11_1,3_4.pkl', 'rb'))
    data_path = "D:/Dataset/PCG_project/My_data/zaoying/all_zaoying/1,3_4/train"
    model_list = [
        "E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/Model/9.21/9.21_5percentage_CL_All_CDG_Model/_folds-7_val_loss-0.26_epoch-04_PCG_model.hdf5",
        "E:/Postgraduate work/Literature/PCG/PCG_Projects/release_demo/Model/9.21/5_percentage/_folds_7val_loss-0.29_epoch-327_ECG_model.hdf5"]
    dataset_test(data_path, model_list, scaler1, scaler2)'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

#颜色 31 红; 32 绿; 33 黄; 34 蓝; 35 洋红; 36 青; 37 白
import re
import numpy as np
import pandas as pd
import scipy.io.wavfile as wav
from python_speech_features import *
import os
import matplotlib.pyplot as plt
from sklearn.model_selection import StratifiedKFold, KFold
from sklearn.preprocessing import StandardScaler,MinMaxScaler
import pickle
import io
import yaml
# from programs_ECG.ECG_features import *
from kmeans_smote import KMeansSMOTE


import warnings
from scipy import signal

def load_csv(open_path):
    with open(open_path, encoding="utf_8_sig") as files:
        my_usecols = list(range(0, 20))
        data = pd.read_csv(files, usecols=my_usecols, header=0, engine="python")
    return data

def read_wave_data(filename):
    """获取语音文件信息： sample_rate:帧速率  signal:数据的矩阵形式 """
    fs, wavsignal = wav.read(filename)  # 声音文件数据的矩阵形式
    return fs, wavsignal

def data_normalization(scaler, raw_train=None, raw_test=None):
    # 若传入scaler则使用保存的归一化参数来归一化
    '''if scaler==False:
        shape_train, shape_test = raw_train.shape, raw_test.shape
        new_train = raw_train.reshape((shape_train[0],-1), order='C')  #展成大向量进行归一化
        new_test = raw_test.reshape((shape_test[0],-1), order='C')
        scaler = StandardScaler()     #Z-score
        train_data = scaler.fit_transform(new_train)   #拟合train的均值和方差
        test_data = scaler.transform(new_test)
        train_data = train_data.reshape(shape_train, order='C')  #reshape成原来的shape
        test_data = test_data.reshape(shape_test, order='C')
        print("\033[32m归一化完成：train_data shape:{:}\033[0m".format(train_data.shape))
        return train_data, test_data , scaler
    else:
        shape_test = raw_test.shape
        scaler = pickle.load(open('scaler_8_30_triple.pkl', 'rb'))
        x_test = raw_test.reshape((shape_test[0], -1), order='C')
        x_test = scaler.transform(x_test)
        x_test = x_test.reshape(shape_test, order='C')  # reshape成原来的shape
        return x_test, scaler'''

    shape_test = raw_test.shape
    # scaler = pickle.load(open('scaler_8_30_triple.pkl', 'rb'))
    x_test = raw_test.reshape((shape_test[0], -1), order='C')
    x_test = scaler.transform(x_test)
    x_test = x_test.reshape(shape_test, order='C')  # reshape成原来的shape
    return x_test, scaler


def _data_normalization(scaler, raw_train=None, raw_test=None):
    # 若传入scaler则使用保存的归一化参数来归一化
    '''if scaler==False:
        shape_train, shape_test = raw_train.shape, raw_test.shape
        new_train = raw_train.reshape((-1,1), order='C')  #展成大向量进行归一化,保留batch
        new_test = raw_test.reshape((-1,1), order='C')
        scaler = StandardScaler()     #Z-score
        train_data = scaler.fit_transform(new_train)   #拟合train的均值和方差
        test_data = scaler.transform(new_test)
        train_data = train_data.reshape(shape_train, order='C')  #reshape成原来的shape
        test_data = test_data.reshape(shape_test, order='C')
        print("\033[32m归一化完成：train_data shape:{:}\033[0m".format(train_data.shape))
        return train_data, test_data , scaler
    else:
        shape_test = raw_test.shape
        scaler = pickle.load(open('scaler_9_11_1,3_4.pkl', 'rb'))
        x_test = raw_test.reshape((-1, 1), order='C')
        x_test = scaler.transform(x_test)
        x_test = x_test.reshape(shape_test, order='C')  # reshape成原来的shape
        return x_test, scaler'''

    shape_test = raw_test.shape
    # scaler = pickle.load(open('scaler_9_11_1,3_4.pkl', 'rb'))
    x_test = raw_test.reshape((-1, 1), order='C')
    x_test = scaler.transform(x_test)
    x_test = x_test.reshape(shape_test, order='C')  # reshape成原来的shape
    return x_test, scaler

def down_resample(data, sample_rate, seconds, ratio):
    new_sample_rate =  int(sample_rate / ratio)
    index = pd.date_range('1/1/2000', periods=sample_rate*seconds, freq='T')  # 这个起始时间任意指定，freq为其频率
    data1 = pd.DataFrame(data)
    data1.index = index
    data_obj = data1.resample(str(ratio)+'T')  # 第一个为抽样频率，label表示左右开闭区间
    data_new = data_obj.asfreq()[0:]
    return np.array(data_new), new_sample_rate

def load_data(root, x, y, section, aug_repeat=1):
    """
    加载数据使其适合网络输入格式
    """
    gap = 7 / (section-1)
    print("\033[32m正在加载wav数据...\033[0m")
    sample_rate = 4000
    x_data = np.zeros((5 * sample_rate, 1))
    y_data = np.zeros((1))
    for i in range(len(x)):
        # print('i为:{},文件是:{}'.format(i,root + "/" + x[i]))
        (sample_rate, signal_read) = wav.read(root + "/" + x[i])
        #此处应有通道选择,取第三通道数据（最显著的）
        # signal_read, sample_rate = down_resample(signal_read[0: int(12*sample_rate),2], sample_rate, 12, 2)  #下采样
        signal_read = signal_read[0:12*sample_rate,2]
        b, a = signal.butter(5, [0.0125, 0.2], btype='bandpass')  #五阶巴特沃斯带通滤波器（通带：25–400赫兹）
        filtedData = signal.filtfilt(b, a, signal_read)  # data为要过滤的信号
        y_data = np.append(y_data, np.repeat(y[i], section))
        x_temp = []
        for j in range(section):
            x_temp.append([])
        for k in range(section):
            x_temp[k] = signal_read[int((k*gap) * sample_rate): int((k*gap+5) * sample_rate)].reshape((-1, 1))   # no filter
            # x_temp[k] = filtedData[int((k*gap) * sample_rate): int((k*gap+5) * sample_rate)].reshape((-1, 1))  # filter
            x_data = np.hstack((x_data, x_temp[k]))
    x_data = np.delete(x_data, 0, axis=1)  # 删掉0列
    y_data = np.delete(y_data, 0)  # 删掉0
    #data augmentation
    if aug_repeat>1:
        x_data_aug, y_data_aug = x_data, y_data
        for i in range(aug_repeat-1):
            #aug_repeat为重复倍数，减去本身的一次
            x_data = np.hstack((x_data, x_data_aug))
            y_data = np.append(y_data, y_data_aug)
    print("\033[32m数据加载完毕...\033[0m")
    return x_data, y_data, sample_rate

def _load_data(root, x, y, section):
    """
    加载数据使其适合网络输入格式,下采样至363Hz,先用2s的不overlap，取5段，10s
    """
    gap = 2
    print("\033[32m正在加载wav数据...\033[0m")
    sample_rate = 4000
    x_data = np.zeros((gap * 363, 1))
    y_data = np.zeros((1))
    for i in range(len(x)):
        # print('i为:{},文件是:{}'.format(i,root + "/" + x[i]))
        (sample_rate, signal_read) = wav.read(root + "/" + x[i])
        #此处应有通道选择,取第三通道数据（最显著的）
        signal_read, sample_rate = down_resample(signal_read[0: int(12*sample_rate),7], sample_rate, 12, 11)  #下采样
        signal_read = signal_read[0:12*sample_rate,0]
        # b, a = signal.butter(5, [0.0125, 0.2], btype='bandpass')  #五阶巴特沃斯带通滤波器（通带：25–400赫兹）
        # filtedData = signal.filtfilt(b, a, signal_read)  # data为要过滤的信号
        y_data = np.append(y_data, np.repeat(y[i], section))
        x_temp = []
        for j in range(section):
            x_temp.append([])
        for k in range(section):
            # x_temp[k] = signal_read[int((k*gap) * sample_rate): int((k*gap+5) * sample_rate)].reshape((-1, 1))   # no filter
            # x_temp[k] = filtedData[int((k*gap) * sample_rate): int((k*gap+5) * sample_rate)].reshape((-1, 1))  # filter
            x_temp[k] = signal_read[int((k * gap) * sample_rate): int(((k+1) * gap) * sample_rate)].reshape((-1, 1))
            x_data = np.hstack((x_data, x_temp[k]))
    x_data = np.delete(x_data, 0, axis=1)  # 删掉0列
    y_data = np.delete(y_data, 0)  # 删掉0
    print("\033[32m数据加载完毕...\033[0m")
    return x_data, y_data, sample_rate

def _Class_Balance(x, y, kmeans):
    print("\033[32m类别平衡中...\033[0m")
    x_data = x.transpose()

    labels, counts = np.unique(y, return_counts=True)
    irt = round((counts[0]+1)/3*(counts[1]+1), 1)
    '''assert (counts[0] > counts[1])  #检查是否0类多于1类
    print("X_shape:{},y_shape{}".format(x_data.shape, y.shape))
    [print('Class {} has {} instances'.format(label, count))
     for label, count in zip(*np.unique(y, return_counts=True))]'''

    with warnings.catch_warnings():
        warnings.filterwarnings('ignore', category=DeprecationWarning)
        warnings.filterwarnings('ignore', category=UserWarning)
        warnings.filterwarnings('ignore', category=RuntimeWarning)
        kmeans_smote = KMeansSMOTE(
            kmeans_args = {'n_clusters': kmeans},   #
            smote_args = {'k_neighbors': 5},        #从少类样本中挑选的个数
            random_state = 2,                       #seed
            sampling_strategy = "minority",
            imbalance_ratio_threshold = 0.2
        )         #只对少类样本重采样
        #imbalance_ratio_threshold配置:小于irt的簇入选
        # ir（不平衡比例）越大说明越不平衡，增大irt会放宽标准，允许更不平衡的簇入选；
        # 降低irt就提高标准，少数类别占比越大的簇才能入选。
        X_resampled, y_resampled = kmeans_smote.fit_sample(x_data, y)

    '''print("X_resmapled_shape:{},y_resmapeled_shape:{},".format(X_resampled.shape, y_resampled.shape))
    [print('Class {} has {} instances after oversampling'.format(label, count))
     for label, count in zip(*np.unique(y_resampled, return_counts=True))]'''
    labels, counts = np.unique(y_resampled, return_counts=True)
    try:
        assert (counts[0] == counts[1])  #检查类别是否已经平衡
    except:
        print('\033[31m类别不平衡:\033[0m')
    # assert (counts[0] == counts[1])  # 检查类别是否已经平衡
    X_resampled = X_resampled.transpose()
    return X_resampled, y_resampled

def _extract_data_by_number(df, number, _class, mode, sub):
    #按照选出的患者的编号将所有的患者的数据取出来。
    # assert ((_class== "abnormal") or (_class=="normal"))
    assert ((mode== "train") or (mode=="test"))
    data  = []
    true_label, label = np.zeros(1), np.zeros(1)
    for i in range(number.shape[0]):
        name = mode + "_" + _class + "_" + str(number[i])
        wav_data = df.loc[df["did"]==name, "waves"]
        local = str(wav_data.values)[1:-1]
        local_1 = re.sub('\'', '', local)
        local_1 = re.sub('\n', ',', local_1)
        local_1 = re.sub(' ', '', local_1)
        local_1 = local_1.split(',')
        data.extend(local_1)
        #获得csv中的真实3类标签(0,1,2)
        this_label = np.array(df.loc[df["did"] == name, "total_result"])  #有多少组数据就有多少个标签，保存最原始的标签
        true_label = np.append(true_label, this_label)
        #判断分类器的类型1(1/3,4->[0][1,2])或者2(1,3/4->[0,1][2])
        for i in range(this_label.shape[0]):   #考虑患者重复的情况
            if sub==1:  #轻度归为异常
                if this_label[i] > 0:
                    label = np.append(label, [1,1,1])
                else:
                    label = np.append(label, [0,0,0])
            elif sub==2:  #轻度归为正常
                if this_label[i] > 1:  label = np.append(label, [1,1,1])
                else: label = np.append(label, [0,0,0])
    label = np.delete(label, 0)
    true_label = np.delete(true_label,0)
    '''if _class == "abnormal":
        label = np.ones(len(data)).astype(int)
    else:
        label = np.zeros(len(data)).astype(int)'''
    return data, label, true_label

def re_distribute_data(k_folds, abnor_num, nor_num):
    '''
    重新分配标签
    :param k_folds: 验证折数
    :param abnor_num: all_train中异常个数
    :param nor_num:  all_train中正常个数
    :return:
    '''
    #在训练集中 K_fold 抽出患者编号，正常和异常分别抽10折
    abnormal_index = np.arange(1,abnor_num+1)
    abnormal_label = np.arange(1,abnor_num+1)
    normal_index = np.arange(1,nor_num+1)
    normal_label = np.arange(1,nor_num+1)
    k_nor_train , k_nor_test, k_abnor_train, k_abnor_test = [],[],[],[]
    kfolder = KFold(n_splits=k_folds, shuffle=False, random_state=0)  #random_state：seed
    for train, test in kfolder.split(abnormal_index, abnormal_label):
        k_abnor_train.append(abnormal_index[train])
        k_abnor_test.append(abnormal_index[test])
    for train, test in kfolder.split(normal_index, normal_label):
        k_nor_train.append(normal_index[train])
        k_nor_test.append(normal_index[test])
    return k_abnor_train, k_abnor_test, k_nor_train, k_nor_test

def _k_folds(data_path, folds, sub):
    for root, dirs, files in os.walk(data_path):
        # 加载label文件
        csv_file = [file for file in files if file.endswith('.csv')]
        csv_data = load_csv(os.path.join(root, csv_file[0]))
        k_abnor_train, k_abnor_test, k_nor_train, k_nor_test = re_distribute_data(folds, 10, 10) #152, 323
        x_train, y_train, x_valid, y_valid, y_train_true, y_valid_true = [], [], [], [], [], []
        for i in range(folds):
            x_train_abnormal, y_train_abnormal, y_train_abnormal_true = _extract_data_by_number(csv_data, k_abnor_train[i], "abnormal", 'train', sub)
            x_valid_abnormal, y_valid_abnormal, y_valid_abnormal_true = _extract_data_by_number(csv_data, k_abnor_test[i], "abnormal", 'train', sub)
            x_train_normal, y_train_normal, y_train_normal_true = _extract_data_by_number(csv_data, k_nor_train[i], "normal", 'train', sub)
            x_valid_normal, y_valid_normal, y_valid_normal_true = _extract_data_by_number(csv_data, k_nor_test[i], "normal", 'train', sub)
            # 顺序必须对应(依次是：训练异常x，训练异常y，训练真实y；验证异常x，验证异常y，验证真实y
            x_train_abnormal.extend(x_train_normal)
            y_train_abnormal = np.append(y_train_abnormal, y_train_normal)
            y_train_abnormal_true = np.append(y_train_abnormal_true, y_train_normal_true)
            x_valid_abnormal.extend(x_valid_normal)
            y_valid_abnormal = np.append(y_valid_abnormal, y_valid_normal)
            y_valid_abnormal_true = np.append(y_valid_abnormal_true, y_valid_normal_true)
            x_train.append(x_train_abnormal)
            y_train.append(y_train_abnormal)
            x_valid.append(x_valid_abnormal)
            y_valid.append(y_valid_abnormal)
            y_train_true.append(y_train_abnormal_true)
            y_valid_true.append(y_valid_abnormal_true)
    return root, x_train, y_train, x_valid, y_valid, y_train_true, y_valid_true


if __name__ == "__main__":
    pass
    # generate_scaler()
    # pickle.dump(2, open('scaler.pkl', 'wb'))
    # train = False
    # if train:
    #     # train
    #     path = 'D:/Dataset/PCG_project/My_data/24+34/train'
    #     to_path = 'D:/Dataset/PCG_project/My_data/24+34/train/_all_train_data.csv'
    #     abnor_num, nor_num = 214, 309
    # else:
    #     # test
    #     path = 'D:/Dataset/PCG_project/My_data/24+34/test'
    #     to_path = 'D:/Dataset/PCG_project/My_data/24+34/test/_all_test_data.csv'
    #     abnor_num, nor_num = 24, 34
    # write_data_to_csv(path, to_path, "test", abnor_num, nor_num)

